# ------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  
# Licensed under the MIT License.  See License in the project root for license information.
# ------------------------------------------------------------------------------

# The SDK version
# x-release-please-start-version
VERSION: str = '1.2.0'
# x-release-please-end
